'''
Created on Dec 17, 2014

@author: marten
'''


import os
import math
import datetime

from sqlalchemy.sql.expression import and_

from django.utils.translation import ugettext as _
from django.core.exceptions import ValidationError
from django.conf import settings

from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.contrib.scheduler.controllers import SchedulerServerController
from basesoft.web.contrib.i18n.models import Country, EFTA_COUNTRIES
from basesoft.web.contrib.license.models import License
from basesoft.web.core.input.validation.plugins import Validation
from basesoft.web.core.util.controllers import AbstractController
from basesoft.web.core.exceptions import PublicSecurityException, PublicUserException

from we.member.models import WeHqppLicense, WeLpp, WeCompany
from we.order.models import WeDeclarationOrder
from we.tbs.models import WeTbsLicense


class UniqueVATNumber(Validation):
    def check(self, params={"db_session": None}):

        if self._value in [None, '']:
            return

        if params['db_session'] is None:
            from basesoft.web.core.util.db import Session
            db_session = Session()
        else:
            db_session = params['db_session']

        _filter = WeCompany.vat_no == self._value

        if params.has_key('company_id') and params['company_id'] != None:
            _filter = and_(_filter, WeCompany.id != params['company_id'])

        lpp_company = db_session.query(WeLpp).join(WeCompany, WeLpp.company_id == WeCompany.id).filter(_filter).first()

        if params.has_key('license_id') and params['license_id'] != None:
            _filter = and_(_filter, License.id != params['license_id'])

        license_company = db_session.query(License).with_polymorphic([WeHqppLicense, WeTbsLicense]).\
            join(WeCompany, License.company_id == WeCompany.id).filter(_filter).first()

        # if this is HQPP check and we have LPP company with that VAT and no other license = pass
        if params['is_hqpp'] is True and lpp_company is not None and license_company is None:
            return
        if lpp_company is not None or license_company is not None:
            raise ValidationError(_('A company is already registered with the provided VAT number.'))


class AbstractWeController(AbstractController):

    def get_months_map(self):
        from basesoft.web.core import L10n_settings as l10n
        return l10n.default.MONTHS_LONG

    def get_months(self):
        months = self.get_months_map()
        rv = []

        for i in xrange(1, len(months) + 1):

            rv.append({
                'value': i,
                'label': months[i - 1]
            })

        return rv

    def get_tbs(self, tbs_id):
        return self.sa_session.query(WeTbsLicense).filter(WeTbsLicense.license_id==tbs_id).one()

    def get_tbs_licenses(self, tbs_id=None):
        """
        This should return the TBS licenses that the current user (member,tbs,operator,accountant) has access to.
        """
        if not self.has_permissions('we_tbs_license'):
            return self._get_own_tbs_licenses(tbs_id)

        q = self.sa_session.query(WeTbsLicense)
        if tbs_id is not None:
            q = q.filter(WeTbsLicense.id == tbs_id)

        rv = q.all()
        if not rv:
            raise PublicSecurityException(_("No such TBS."))

        return rv

    def _get_own_tbs_licenses(self, tbs_id=None):
        """
        This will return the tbs licenses where the current user is part of.
        """
        rv = []
        for group in self.get_active_groups():
            if not group.license.is_active:
                continue
            if isinstance(group.license, WeTbsLicense):
                if group.license not in rv and (tbs_id is None or tbs_id == group.license.id):
                    rv.append(group.license)

            elif isinstance(group.license, WeHqppLicense):
                lpps = self.get_local_producer_partners(group.license.id)
                for lpp in lpps:
                    if lpp.tbs not in rv and (tbs_id is None or tbs_id == lpp.tbs.id):
                        rv.append(lpp.tbs)
            # TODO: check for accountant, lpp etc.

        if not rv:
            raise PublicSecurityException(_("No access to TBS."))

        return rv

    def get_member_licenses(self, member_id=None):
        """
        If user doesn't have 'we_create_member_license' he is not an operator and fetch only own licenses
        else fetch all member licenses and if member_id is not None fetch that license
        :param member_id: int
        :return: List of licenses
        """

        if not self.has_permissions('we_create_member_license'):
            return self._get_own_member_licenses(member_id)

        q = self.sa_session.query(WeHqppLicense)
        if member_id is not None:
            q = q.filter(WeHqppLicense.id==member_id)
        return q.all()

    def _get_own_member_licenses(self, member_id=None):
        """
        This will return the member licenses where the current user is part of.
        """
        if isinstance(self.get_default_group().license, WeTbsLicense):
            return self._get_tbs_member_licenses(member_id)
        rv = []
        for group in self.get_active_groups():
            if not group.license.is_active:
                continue
            if not isinstance(group.license, WeHqppLicense):
                continue
            if group.license not in rv and (member_id is None or member_id == group.license.id):
                rv.append(group.license)

        if not rv:
            raise PublicSecurityException(_("No access to HQPP."))

        return rv

    def _get_tbs_member_licenses(self, member_id=None):
        self.assert_permissions('we_view_lpp_companies')
        assert isinstance(self.get_default_group().license, WeTbsLicense)

        q = self.sa_session.query(WeHqppLicense).join(WeLpp)
        _filter = WeLpp.tbs == self.get_default_group().license
        if member_id is not None:
            _filter = and_(_filter, WeHqppLicense.id == member_id)
        members = q.filter(_filter).all()
        if member_id is not None and len(members) == 0:
            raise PublicSecurityException(_("No access to HQPP."))
        return members

    def get_local_producer_partners(self, member_id=None, status=WeLpp.STATUS_ACTIVE, tbs_id=None):
        """
        This will return the local producer partners a logged in user has access to, for the supplied member_id and status.
        TODO: limit LPPs per account (#12)
        """

        members = self.get_member_licenses(member_id)

        is_tbs = isinstance(self.get_default_group().license, WeTbsLicense)

        rv = []
        for member in members:
            for lpp in member.lpps:
                if tbs_id is not None and lpp.tbs.id != tbs_id:
                    continue
                if is_tbs and lpp.tbs not in [g.license for g in self.get_active_groups()]:
                    continue
                if status == lpp.status:
                    rv.append(lpp)
        return rv

    def get_local_producer_partner(self, lpp_id):
        lpp = self.sa_session.query(WeLpp).filter(WeLpp.id == lpp_id).first()
        if lpp is None:
            return None
        members = self.get_member_licenses(lpp.hqpp.id)
        tbses = self.get_tbs_licenses()

        if lpp.hqpp not in members or lpp.tbs not in tbses:
            raise PublicSecurityException(_('No access to local producer partner'))

        return lpp

    def get_lpp_with_all_tbses(self, company_id):
        """
        An LPP can be registered in multiple TBSes so we should use Company id for correct distinction
        and return all active records for that Company
        :param company_id: int
        :return: List
        """
        lpp = self.sa_session.query(WeLpp).filter(WeLpp.company_id == int(company_id)).all()
        if len(lpp) == 0:
            return []

        records = []
        members = self.get_member_licenses()

        for lp in lpp:
            if lp.hqpp in members:
                records.append(lp)

        if len(records) == 0:
            raise PublicUserException(_('No access to local producer partner.'))
        return records

    def assert_local_producer_partner(self, lpp_id):
        lpp = self.get_local_producer_partner(lpp_id)
        if lpp is None:
            raise PublicSecurityException(_('No access to local producer partner'))

    def get_file_extension(self, file_name, file_hash):

        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, file_hash)
        assert os.path.exists(file_path), 'file not found: %s' % file_path

        return os.path.splitext(file_name)

    def start_task(self, task, params):
        ssc = SchedulerServerController(self.sa_session)
        ssc.system_start_task(task, params)

    def get_tbs_period_for_day(self, tbs, day):
        # check tbs setting declaration period end day

        periodicity = tbs.get_config('we.tbs.declaration.period')

        td = datetime.timedelta(days=day-1)

        today = tbs.domain.today()

        y = today.year
        d = []
        d.append(datetime.date(y, 1, 1)+td)  # every periodicity has this one.
        month = None
        quarter = None
        biannual = None
        year = today.year

        # DECLARATION PERIODS are not the same as POM PERIODS

        if periodicity != WeTbsLicense.DECLARATION_PERIOD_YEARLY:
            d.append(datetime.date(y, 7, 1)+td)  # bi-annual and others.
            biannual = WeDeclarationOrder.BI_ANNUAL_SECOND_HALF if today.month < 7 else WeDeclarationOrder.BI_ANNUAL_FIRST_HALF
            if periodicity == WeTbsLicense.DECLARATION_PERIOD_BIANNUALLY and today.month < 7:
                year = today.year - 1
        else:
            year = today.year - 1

        if periodicity not in (WeTbsLicense.DECLARATION_PERIOD_YEARLY, WeTbsLicense.DECLARATION_PERIOD_BIANNUALLY, ):
            d.append(datetime.date(y, 4, 1)+td)  # quarterly and monthly
            d.append(datetime.date(y, 10, 1)+td)  # quarterly and monthly
            biannual = None

        # for monthly add the remaining dates
        if periodicity == WeTbsLicense.DECLARATION_PERIOD_MONTHLY:
            month = today.month - 1
            if month == 0:
                month = 12
                year = today.year-1

            d.append(datetime.date(y, 2, 1)+td)
            d.append(datetime.date(y, 3, 1)+td)
            d.append(datetime.date(y, 5, 1)+td)
            d.append(datetime.date(y, 6, 1)+td)
            d.append(datetime.date(y, 8, 1)+td)
            d.append(datetime.date(y, 9, 1)+td)
            d.append(datetime.date(y, 11, 1)+td)
            d.append(datetime.date(y, 12, 1)+td)

        if periodicity == WeTbsLicense.DECLARATION_PERIOD_QUARTERLY:
            quarter = int(math.floor((today.month-1)/3))
            if quarter == 0:
                quarter = 4
                year = today.year-1

        if today not in d:
            return None

        return year, biannual, quarter, month

    def get_tbs_period_for_month(self, tbs, month, year):

        periodicity = tbs.get_config('we.tbs.declaration.period')

        y = year
        m = None
        quarter = None
        biannual = None

        if periodicity == WeTbsLicense.DECLARATION_PERIOD_BIANNUALLY:
            biannual = WeDeclarationOrder.BI_ANNUAL_FIRST_HALF if month < 7 else WeDeclarationOrder.BI_ANNUAL_SECOND_HALF

        if periodicity == WeTbsLicense.DECLARATION_PERIOD_MONTHLY:
            m = month

        if periodicity == WeTbsLicense.DECLARATION_PERIOD_QUARTERLY:
            quarter = int(month / 3)
            if month not in [3, 6, 9, 12]:
                quarter += 1
        return y, biannual, quarter, m

    def get_months_for_period(self, month=None, quarter=None, biannual=None):
        quarter = None if quarter == 0 else quarter
        month = None if month == 0 else month
        biannual = None if biannual == 0 else biannual

        # get all exported declarations within the provided period for the Take-back scheme.
        if month is not None:
            assert quarter is None
            assert biannual is None
            months = [month]
        elif quarter is not None:
            assert biannual is None
            months = range(((quarter-1)*3)+1, (quarter*3)+1)
        elif biannual is not None:
            months = range(1, 7) if biannual == WeDeclarationOrder.BI_ANNUAL_FIRST_HALF else range(7, 13)
        else:
            months = range(1, 13)

        return months

class WeCoreController(AbstractWeController):

    def get_efta_countries(self):
        return self.sa_session.query(Country).filter(Country.iso_3166_a2.in_(EFTA_COUNTRIES)).all()


class WeNotificationController(NotificationController):
    pass
