# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from sqlalchemy.orm.exc import NoResultFound

from basesoft.web.contrib.branding.models import Site, Theme
from basesoft.web.contrib.license.models import Domain
from basesoft.web.contrib.branding.db_sync import _sync_themes

def post_sync(sa_session, app_name):
    """
    when weee-eu is loaded, syncdb updates the default site to weee-eu
    """
    if app_name == 'basesoft.web.contrib.branding':

        # sync themes only after color_sets have been created
        themes = [{
            'name': u'WEEE Europe',
            'title': u'WEEE Europe',
            'module_name': u'weee_europe_theme_default',
            'css_class': u'weeee',
            'licenses': [],
            'default_graph_theme': u'Default',
        }]
        _sync_themes(sa_session, themes)

        #
        # setup existing default Site object with pvcycle domain/license settings
        #

        # get/create weee europe domain
        try:
            domain = sa_session.query(Domain).filter(Domain.name == 'WEEE Europe').one()
        except NoResultFound:
            domain = Domain()
            domain.name = 'WEEE Europe'
            domain.settings = 0
        # when a user registers for the domain he becomes the admin for the new member
        sa_session.add(domain)

        if settings.DEBUG:
            # set new public home app
            site = sa_session.query(Site).filter(Site.host_match=='').one()
        else:
            try:
                site = sa_session.query(Site).filter(Site.host_match=='.*weee-eu.*').one()
            except NoResultFound:
                site = Site()
                site.host_match = '.*weee-eu.*'
                site.order = 997
                site.trac_conn_str = 'http://'

        site.home_app = 'we'
        site.domain = domain
        site.theme = sa_session.query(Theme).filter(Theme.name=='WEEE Europe').one();
        sa_session.add(site)
        sa_session.flush()

