'''
'''
from datetime import datetime
import decimal
from basesoft.web.contrib.order.soap import register_soap_functions as iqportal_order_functions
from basesoft.web.contrib.soap import soap_assert_logged_in

from basesoft.web.contrib.soap.args import _types

from pyws.functions.register import register
from basesoft import logging


# Declaration Data
from we.core.controllers import AbstractWeController
from we.order.controllers import WeDeclarationOrderController

DeclarationLineDataDict = {
    0: 'DeclarationLineDataDict',
    'position': int,
    'local_category': str,
    'quantity': int,
    'weight': decimal.Decimal,
    'created': datetime,
    'last_updated': datetime,
    'properties': [_types.KeyValueDict],
}

DeclarationDataDict = {
    0: 'DeclarationDataDict',
    'id': int,
    'lpp_relation_id': str, # str until we find out what is this exactly
    'invoice_company_relation_id': str,
    'year': int,
    'month': int,
    'quarter': int,
    'bi-annual': int,
    'lines': [DeclarationLineDataDict],
    'created': datetime,
    'last_updated': datetime,
    'properties': [_types.KeyValueDict],
}

def register_soap_functions(server):

    @register(return_type=[_types.LabelValueDict], to=server, needs_context=True)
    def get_take_back_schemes(context):
        """
        Returns a list with the ID and the Name of each TBS linked to the current logged in user.
        """

        soap_assert_logged_in(context=context)

        logger = logging.getLogger('weee-eu.core.soap', request=context)
        logger.info('get_take_back_schemes')
        ac = AbstractWeController(context.Session(), request=context, account_id=context.session['user_id'])
        return [{'id': t.id, 'name': t.name} for t in ac.get_tbs_licenses()]

    @register(return_type=[_types.LabelValueDict], to=server, needs_context=True)
    def get_members(context):
        """
        Returns a list with the ID and the Name of each HQPP linked to the current logged in user.
        """

        soap_assert_logged_in(context=context)

        logger = logging.getLogger('weee-eu.core.soap', request=context)
        logger.info('get_members')
        ac = AbstractWeController(context.Session(), request=context, account_id=context.session['user_id'])
        return [{'id': t.id, 'name': t.name} for t in ac.get_member_licenses()]


    # add iqportal order methods
    iqportal_order_functions(server, WeDeclarationOrderController)
