# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.


from django.utils.translation import ugettext as _
from sqlalchemy.sql.expression import and_

from basesoft.scheduler.crontab import CronTab
from basesoft.scheduler.task import ScheduleTask
from basesoft.web.contrib.file.schedule import AbstractImportFileTask
from basesoft.web.contrib.notification.controllers import NotificationController

from iqportal.iqube.core.schedule import AbstractAggregateTask

from we.declaration.controllers import WeDeclarationController
from we.declaration.importers import WeDeclarationAbstractImporter
from we.declaration.models import WeDeclaration
from we.member.models import WeHqppLicense
from we.order.models import WeDeclarationOrder


class WeDeclarationImportTask(AbstractImportFileTask):
    """
    Background action for importing saved files with LPP declarations.
    """
    INTERVAL = 0
    QUEUE_IF_EXISTS = True
    controller = WeDeclarationController
    importer = WeDeclarationAbstractImporter


class WeDeclarationReminderTask(ScheduleTask):
    # runs every day at 00:20
    INTERVAL = CronTab('20 0 * * *')

    def execute(self):
        hqpps = self.sa_session.query(WeHqppLicense).filter(WeHqppLicense.is_active==True).all()
        dc = WeDeclarationController(self.sa_session)

        for hqpp in hqpps:
            lpps = []
            for lpp in hqpp.lpps:
                tbs = lpp.tbs
                reminder_day = tbs.get_config('we.tbs.declaration.reminder.day')

                vals = dc.get_tbs_period_for_day(tbs, reminder_day)
                if vals is None:
                    continue

                year = vals[0]
                biannual = vals[1]
                quarter = vals[2]
                month = vals[3]

                if month is not None:
                    months = [month]
                elif quarter is not None:
                    months = range(((quarter-1)*3)+1, (quarter*3)+1)
                elif biannual is not None:
                    months = range(1, 7) if biannual == WeDeclarationOrder.BI_ANNUAL_FIRST_HALF else range(7, 13)
                else:
                    months = range(1, 13)

                # get all unprocessed declarations per lpp,
                declarations = self.sa_session.query(WeDeclaration).filter(
                    and_(
                        WeDeclaration.lpp == lpp,
                        WeDeclaration.status == WeDeclaration.STATUS_COMPLETE,
                        WeDeclaration.declaration_order == None,
                        WeDeclaration.year == year,
                        WeDeclaration.month.in_(months)
                    )
                ).order_by(WeDeclaration.month).all()

                if len(declarations) > 0:
                    self.logger.info(' - At least one Declaration found for LPP ID[%s]. No notification created.' % lpp.id)
                    continue
                lpps.append(lpp.name)

            if len(lpps) > 0:
                dc.create_declaration_reminder_notification(hqpp)
                self.logger.info(' - No Declarations found. Notification created.')


tasks = (WeDeclarationImportTask, WeDeclarationReminderTask, )