# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy import func, and_

from django.utils.translation import ugettext as _

from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.order.custom_query import AbstractOrderCustomQuery, Merchant
from basesoft.web.contrib.order.models import Currency, InvoiceFile
from basesoft.web.core.util.custom_query import CustomQueryModel, CustomQueryField

from we.core.custom_query import AbstractWeCustomQuery
from we.member.models import WeLpp
from we.order.models import WeDeclarationOrder


class WeDeclarationOrderCustomQuery(AbstractOrderCustomQuery, AbstractWeCustomQuery):
    permissions = ['we_view_declaration_order']

    def tables(self):
        return (
            CustomQueryModel(WeDeclarationOrder),
            CustomQueryModel(WeLpp, onclause=WeDeclarationOrder.lpp_id == WeLpp.id),
            CustomQueryModel(License, onclause=WeDeclarationOrder.client_id == License.id),
            CustomQueryModel(Merchant, onclause=WeDeclarationOrder.merchant_id == Merchant.id),
            CustomQueryModel(Currency, onclause=WeDeclarationOrder.currency_id == Currency.id),
            CustomQueryModel(InvoiceFile, onclause=WeDeclarationOrder.invoice_id == InvoiceFile.file_id,
                             use_outerjoin=True, priv=None),
        )

    def fields(self):
        return (
            CustomQueryField(WeDeclarationOrder.id, 'Order ID', key='id', is_filter=True),
            CustomQueryField(WeDeclarationOrder.polymorphic_type, 'Polymorphic type', key='polymorphic_type'),
            CustomQueryField(WeDeclarationOrder.id, _('Details'), key='details'),
            CustomQueryField(WeDeclarationOrder.year, _('Year'), key='year', is_filter=True, filter_options=WeDeclarationOrder.year),
            CustomQueryField(WeDeclarationOrder.month, _('Month'), key='month', is_filter=True, filter_options=WeDeclarationOrder.month),
            CustomQueryField(WeDeclarationOrder.quarter, _('Quarter'), key='quarter', is_filter=True, filter_options=WeDeclarationOrder.quarter),
            CustomQueryField(WeDeclarationOrder.status, _('Status'), key='status', is_filter=True, filter_options=[{
                'value': WeDeclarationOrder.STATUS_NEW,
                'label': _("New"),
            },{
                'value': WeDeclarationOrder.STATUS_PENDING,
                'label': _("Declared"),
            },{
                'value': WeDeclarationOrder.STATUS_SUBMITTED,
                'label': _("Submitted"),
            },{
                'value': WeDeclarationOrder.STATUS_CREDITED,
                'label': _("Credited"),
            }]),
            CustomQueryField(WeDeclarationOrder.type, _('Type'), key='type', is_filter=True),
            # CustomQueryField(WeDeclarationOrder.resubmit_request, _('Resubmit request'), key='resubmit_request', is_filter=True),
            CustomQueryField(WeDeclarationOrder.export_date, _('Export date'), key='export_date'),
            CustomQueryField(License.name, _('Producer'), key='client', is_filter=True),
            CustomQueryField(WeLpp.name, _('LPP'), key='lpp', is_filter=True),
            CustomQueryField(Merchant.name, _('TBS'), key='merchant', is_filter=True, is_group_by=True),
            CustomQueryField(func.ifnull(Currency.symbol, Currency.code), _('Currency'), 'currency'),
            CustomQueryField(WeDeclarationOrder.total_amount_excluding_vat, _('Total amount (ex VAT)'), 'total_amount'),
            CustomQueryField(WeDeclarationOrder.total_amount_including_vat, _('Total amount (incl VAT)'), 'total_amount_incl_vat'),
            CustomQueryField(WeDeclarationOrder.status, _('Action'), key='action'),
            CustomQueryField(WeDeclarationOrder.id, _('Order log'), key='order_log'),
        )

    def get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count=True):
        if len(applied_filters) == 0 and not self.separated:
            if self.where is None:
                self.where = WeDeclarationOrder.child_order == None
            else:
                self.where = and_(self.where, WeDeclarationOrder.child_order == None)

        return AbstractOrderCustomQuery.get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count)

    group_by = (WeDeclarationOrder.id,)


custom_queries = (
    WeDeclarationOrderCustomQuery,
)
