# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import event, SmallInteger, Integer, Numeric
from sqlalchemy.orm import mapper, relation
from sqlalchemy.sql.schema import ForeignKey, Column
from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models


__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    tblWeDeclarationOrder = IQTable('tblWeDeclarationOrder', metadata,
        Column('order_id', ForeignKey('tblOrder.id', ondelete='CASCADE'), primary_key=True),
        Column('lpp_id', Integer, ForeignKey('tblWeLpp.id', ondelete='RESTRICT'), nullable=False),

        Column('resubmit_request', SmallInteger),
        Column('year', SmallInteger, nullable=False),
        Column('month', SmallInteger),
        Column('quarter', SmallInteger),
        Column('bi_annual', SmallInteger),
    )

    tblWeDeclarationOrderLine = IQTable('tblWeDeclarationOrderLine', metadata,
        Column('order_line_id', Integer, ForeignKey('tblOrderLine.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_product_level_id', Integer, ForeignKey('tblWeTbsProductLevel.id', ondelete='RESTRICT'), nullable=False),
        Column('weight', Numeric(20, 5), nullable=False),
        Column('is_b2b', SmallInteger, nullable=False), # 0 = Not supplied, 1 = b2c, 2 = b2b
    )

    tables['tblWeDeclarationOrder'] = tblWeDeclarationOrder
    tables['tblWeDeclarationOrderLine'] = tblWeDeclarationOrderLine


    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from we.order import models
    from we.order.hooks import we_declaration_order_listener
    from basesoft.web.contrib.order import models as order_models
    from we.member import models as member_models
    from we.product import models as product_models


    mapper(models.WeDeclarationOrder, tables['tblWeDeclarationOrder'],
        inherits=order_models.Order,
        properties = {
            'lpp': relation(member_models.WeLpp),
        },
        polymorphic_identity='WeDeclarationOrder'
    )

    event.listen(models.WeDeclarationOrder, 'after_insert', we_declaration_order_listener)

    mapper(models.WeDeclarationOrderLine, tables['tblWeDeclarationOrderLine'],
        inherits=order_models.OrderLine,
        properties = {
            'tbs_product_level': relation(product_models.WeTbsProductLevel),
        },
        polymorphic_identity='WeDeclarationOrderLine'
    )

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'WEEE Europe Order')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [
        license_models.Permission(u'we_view_declaration_order', u"""
            This gives the right to view WEEE-EU Declaration orders.
            """, _("View WEEE-EU declaration orders")),

        license_models.Permission(u'we_manage_declaration_order', u"""
            This gives the right to manage WEEE-EU Declaration orders.
            """, _("Manage WEEE-EU declaration orders")),

    ]