# -*- coding: UTF-8 -*-
'''
Created on Oct 10, 2014

@author: rado
'''

from django.utils.translation import ugettext, ugettext_lazy as _

from sqlalchemy.orm.session import Session
from sqlalchemy.sql.expression import and_

from basesoft.web.contrib.notification.models import Notification

from we.tbs.models import WeTbsRegulationPeriod


class WeCN8(object):
    """
    @DynamicAttrs
    """

    def __repr__(self):
        return '<WeCN8 id=%s>' % self.code

class WeUnuCode(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<WeUnuCode id=%s, code=%s >' % (self.id, self.code)

class WeTbsProductLevel(object):
    """
    @DynamicAttrs
    """

    INVOICE_BASED_ON_WEIGHT = 1
    INVOICE_BASED_ON_QUANTITY = 2
    INVOICE_BASED_ON_FIXED_ANNUAL_FEE = 3

    def get_current_regulation_period_product(self, regulation_period_id=None):
        session = Session.object_session(self)
        _filter = and_(WeTbsRegulationPeriodProduct.tbs_product_level_id == self.id,
                       WeTbsRegulationPeriod.archived == 0)

        if regulation_period_id is not None:
            _filter = and_(_filter, WeTbsRegulationPeriod.id == regulation_period_id)
        else:
            _filter = and_(_filter, WeTbsRegulationPeriod.start_date <= self.tbs.domain.today())

        return session.query(WeTbsRegulationPeriodProduct).join(WeTbsRegulationPeriod)\
            .filter(_filter).order_by(WeTbsRegulationPeriod.start_date.desc()).first()

    current_regulation_period_product = property(get_current_regulation_period_product)

    def __repr__(self):
        return '<WeTbsProductLevel id=%s label="%s">' % (self.id, self.label)


class WeTbsRegulationPeriodProduct(object):
    """
    @DynamicAttrs
    """
    def get_state(self):
        pl = self.tbs_product_level
        return {
            'id': self.id,
            'tbs_product_level_id': pl.id,
            'label': pl.label,
            'description': pl.description,
            'fee': self.fee
        }

    def __repr__(self):
        return '<WeTbsRegulationPeriodProduct id=%s, position=%s>' % (self.id, self.position)


class WeCN8PeriodProduct(object):
    """
    @DynamicAttrs
    """

    B2B_NOT_APPLICABLE = 0
    IS_B2B_FALSE = 1
    IS_B2B_TRUE = 2

    def __repr__(self):
        return '<WeCN8PeriodProduct cn8_code=%s, product_level=%s, ' \
               'is_b2b=%s>' % (self.cn8_code, self.tbs_product_level.label, self.is_b2b)


class WeHqppProduct(object):
    """
    @DynamicAttrs
    """

    def get_period_product(self, regulation_period, is_b2b=False):
        # deprecated
        session = Session.object_session(self)
        _filter = WeHqppRegulationPeriodProduct.product_id == self.id
        _filter = and_(_filter, WeHqppRegulationPeriodProduct.tbs_regulation_period == regulation_period)

        q = session.query(WeHqppRegulationPeriodProduct)

        if not is_b2b:
            return q.filter(and_(_filter, WeHqppRegulationPeriodProduct.is_b2b == WeHqppRegulationPeriodProduct.B2B_NOT_APPLICABLE)).all()

        return q.filter(and_(_filter, WeHqppRegulationPeriodProduct.is_b2b != WeHqppRegulationPeriodProduct.B2B_NOT_APPLICABLE)).all()

    def get_total_weight_per_unit_for_regulation_period(self, regulation_period):

        _map = {
            regulation_period.WEIGHT_PER_UNIT: 'weight_per_unit',
            regulation_period.WEIGHT_CATEGORY_1: 'weight_cat1',
            regulation_period.WEIGHT_CATEGORY_2: 'weight_cat2',
            regulation_period.WEIGHT_CATEGORY_3: 'weight_cat3',
            regulation_period.WEIGHT_CATEGORY_4: 'weight_cat4',
            #regulation_period.WEIGHT_CATEGORY_5: '',  # FIXME, this is listed in the excel (ticket #350)  but not on the product sheet?
            regulation_period.WEIGHT_DOCUMENTATION: 'weight_documentation',
            regulation_period.WEIGHT_BATTERIES: 'weight_batteries',
            regulation_period.WEIGHT_PACKAGING: 'weight_package',
        }

        total = 0

        for key, field in _map.iteritems():
            if regulation_period.weight_settings & key != key:
                continue
            value = getattr(self, field)
            if value is None:
                continue
            total += value

        return total

    def __repr__(self):
        return "<WeHqppProduct id=%s hqpp_id=%s sku=%s>" % (self.id, self.hqpp_id, self.sku)


class WeHqppRegulationPeriodProduct(object):
    """
    @DynamicAttrs
    """
    B2B_NOT_APPLICABLE = 0
    IS_B2B_FALSE = 1
    IS_B2B_TRUE = 2

    def get_end_user_distinction(self):
        if self.is_b2b == self.B2B_NOT_APPLICABLE:
            return None
        elif self.is_b2b == self.IS_B2B_FALSE:
            return 'B2C'
        else:
            return 'B2B'

    def __repr__(self):
        return "<WeHqppRegulationPeriodProduct id=%s product_id=%s " \
               "regulation period=%s is_b2b=%s>" % (self.id, self.product_id, self.tbs_regulation_period_id, self.is_b2b)


class WeHqppProductExcludedAtTbs(object):
    pass


class WeRegulationPeriodNotification(Notification):
    name_i18n = _('Regulation period')

    def get_message(self):
        return ugettext('Not all of your products could be automatically reassigned for the new Regulation Period, please use the Perform Action button to update your products.')

    def get_subject(self):
        return ugettext('New regulation period - action required')

    def __repr__(self):
        return '<WeRegulationPeriodNotification id=%s>' % self.id
