'''
Created on Oct 14, 2014

@author: rado
'''
import os

from django.http import HttpResponse

from django.conf import settings

from we.product.controllers import WeProductController
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from django.utils.translation import ugettext as _

from we.product.importers import CN8AbstractImporter, HqppProductAbstractImporter, \
    TbsPriceSheetAbstractImporter, CN8ToTbsProductLevelAbstractImporter


class WeProductRpcHandler:

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']
        self.wpc = WeProductController(self.sa_session, account_id=self.account_id, request=request)

    def get_all_months(self):
        return self.wpc.get_months()

    def get_all_available_months_for_year(self, tbs_id, year=None):
        if tbs_id in ['', None] or year is None:
            return self.get_all_months()

        return self.wpc.get_available_months_in_year(tbs_id, year)

    def get_weee_product_levels(self):
        return self.wpc.get_weee_product_levels()

    def get_cn8_codes(self, selected, filter_text, limit=100):
        return self.wpc.get_cn8_codes(selected, filter_text, limit)

    def get_available_cn8_codes(self, selected, filter_text, limit=100, get_available=True):
        return self.wpc.get_cn8_codes(selected, filter_text, limit, get_available)

    def get_unu_codes(self):
        return self.wpc.get_unu_codes()

    def save_or_update_cn8_weee_product_level(self, _id, state):
        return self.wpc.save_or_update_cn8_weee_product_level(_id, state).id

    def load_cn8_weee_product_level(self, product_level_id):
        return self.wpc.load_cn8_weee_product_level(product_level_id)

    def import_cn8_codes(self, file_hash):
        tmp_file = file_hash[0]
        importer = CN8AbstractImporter().get_importer(tmp_file)(self.sa_session, account_id=self.account_id)
        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, tmp_file.file_hash)

        if importer.get_row_count(file_path) > importer.MAX_RECORDS_FOR_INSTANT_IMPORT:
            importer.import_file(tmp_file, None)
            self.wpc.start_task('we.product.schedule.CN8ToWeeeCategoryTask', {})
            return False

        return self.wpc.import_cn8_codes(tmp_file, importer)

    def save_or_update_cn8_code(self, state):
        self.wpc.save_or_update_cn8_code(state)

    def validate_cn8_codes(self, product_level_id, cn8_codes):
        return self.wpc.validate_cn8_codes(product_level_id, cn8_codes)

    def delete_cn8_code_matches(self, codes):
        self.wpc.delete_cn8_code_matches(codes)

    def save_tbs_product_level(self, _id, values):
        self.wpc.save_or_update_tbs_product_level(_id, **values)
        return True

    def load_tbs_product_level(self, _id, regulation_period_id):
        pl = self.wpc.get_tbs_product_levels(_id)
        if pl is None:
            return {}

        rpp = pl.get_current_regulation_period_product(regulation_period_id)
        return {
            'tbs_id': pl.tbs.id,
            'parent_level_id': pl.parent_level_id,
            'label': pl.label,
            'description': pl.description,
            'invoice_basis': pl.invoice_basis,
            'fee': rpp.fee,
            'tbs_regulation_period_id': rpp.tbs_regulation_period.id
        }

    def get_parent_category_for_tbs(self, tbs_id, regulation_period_id):
        return self.wpc.get_parent_category_for_tbs(tbs_id, regulation_period_id)

    def get_tbs_product_levels(self, tbs_id):
        options = []
        categories = self.wpc.get_tbs_product_levels(None, tbs_id=tbs_id)
        for cat in categories:
            options.append({
                'label': '%s - %s' % (cat.label, cat.description),
                'value': cat.id
            })

        return options

    def get_product_details(self, ids, rp_id):
        return self.wpc.get_product_details(ids, rp_id)

    def get_product_sheet_tbs_product_levels(self, tbs_id, cn8, period=None, is_b2b=None):
        options = [{
            'label': _('Suggested categories'),
            'id': 'suggested_categories',
            'options': []
        }]

        suggested = self.wpc.get_suggested_tbs_product_levels(tbs_id, cn8, period, is_b2b)

        for c in suggested:
            options[0]['options'].append({
                'label': '%s - %s' % (c.label, c.description),
                'value': c.id
            })

        categories = self.wpc.get_tbs_product_levels_for_period(tbs_id, period, include_null_fees=False)

        options.append({
            'label': _('All categories'),
            'id': 'all_categories',
            'options': []
        })
        for cat in categories:
            options[1]['options'].append({
                'label': '%s - %s' % (cat.label, cat.description),
                'value': cat.id
            })

        return options

    def load_regulation_period(self, _id):
        return self.wpc.load_regulation_period_state(_id)

    def save_regulation_period(self, _id, values):
        rp = self.wpc.save_or_update_regulation_period(_id, **values)
        return rp.id

    def get_regulation_period(self, _id):
        rp = self.wpc.get_regulation_period_by_id(None, _id)
        return {
            'tbs_id': rp.tbs.id,
            'tbs_name': rp.tbs.name,
            'start_date': rp.start_date,
            'label': rp.label,
            'title': rp.title,
            'is_processed': rp.is_processed,
            'has_end_user_distinction': rp.tbs.get_config('we.tbs.require.end.user.distinction')
        }

    def get_regulation_period_comparison(self, _id):
        rp = self.get_regulation_period(_id)
        rv = {
            'regulation_period': rp,
            'differences': self.wpc.compare_regulation_period_with_previous(_id, dry_run=True),
        }
        return rv

    def get_tbs_regulation_periods(self, tbs_id):
        rv = self.wpc.get_regulation_periods(tbs_id)
        options = []
        for r in rv:
            options.append({
                'label': r.title,
                'value': r.id
            })
        return options

    def save_or_update_hqpp_product(self, product_id, state):
        return self.wpc.save_or_update_hqpp_product(product_id, state, state['declaration_period']).id

    def save_hqpp_regulation_period_product(self, product_id, regulation_period_id, tbs_product_level_id, is_b2b):
        self.wpc.save_or_update_hqpp_regulation_period_product(product_id, regulation_period_id, tbs_product_level_id, is_b2b)
        return True

    def load_hqpp_product(self, product_id):
        return self.wpc.load_hqpp_product(product_id)

    def get_hqpp_product_tbs_links(self, member_id, product_id, declaration_period=None, cn8_code=None):
        return self.wpc.get_hqpp_product_tbs_links(member_id, product_id, declaration_period, cn8_code)

    def get_skus(self, contains=None, lpp_id=None, exclude=None):
        return [p.sku for p in self.wpc.get_skus(lpp_id, contains, exclude)]

    def get_product_info(self, lpp_id, sku, year, month):
        return self.wpc.get_product_info(lpp_id, sku, year, month)

    def import_hqpp_product_sheet(self, file_hash, hqpp_id, regulation_periods_state):
        self.wpc.assert_permissions('we_create_products')
        tmp_file = file_hash[0]
        importer = HqppProductAbstractImporter.get_importer(tmp_file)(self.sa_session, account_id=self.account_id)
        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, tmp_file.file_hash)

        if importer.get_row_count(file_path) > importer.MAX_RECORDS_FOR_INSTANT_IMPORT:
            importer.import_file(tmp_file, None, hqpp_id=hqpp_id,
                                 regulation_periods=regulation_periods_state)
            self.wpc.start_task('we.product.schedule.HqppProductSheetTask', {})
            return False

        return self.wpc.import_hqpp_product_sheet(tmp_file, importer,
                                                  hqpp_id, regulation_periods_state)

    def import_tbs_price_sheet_file(self, file_hash, tbs_id, regulation_period_id, regulation_label=None,
                                    regulation_year=None, regulation_month=None, weight_settings=None):
        self.wpc.assert_permissions('we_manage_tbs_product_level')
        tmp_file = file_hash[0]
        importer = TbsPriceSheetAbstractImporter().get_importer(tmp_file)(self.sa_session, account_id=self.account_id)
        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, tmp_file.file_hash)
        row_count = importer.get_row_count(file_path)

        if row_count > importer.MAX_RECORDS_FOR_INSTANT_IMPORT:
            importer.import_file(tmp_file, importer, tbs_id=tbs_id, regulation_period_id=regulation_period_id,
                                 regulation_label=regulation_label, regulation_year=regulation_year,
                                 regulation_month=regulation_month, row_count=row_count,
                                 account_id=self.account_id, weight_settings=weight_settings)
            self.wpc.start_task('we.product.schedule.TbsPriceSheetImportTask', {})
            return False

        return self.wpc.import_tbs_price_sheet(tmp_file, importer, tbs_id, row_count,
                                               regulation_period_id, regulation_label,
                                               regulation_year, regulation_month, weight_settings=weight_settings)

    def import_cn8_to_tbs_product_level(self, file_hash, tbs_id, regulation_period_id):
        self.wpc.assert_permissions('we_manage_cn8_tbs_product_level')
        tmp_file = file_hash[0]
        importer = CN8ToTbsProductLevelAbstractImporter().get_importer(tmp_file)(self.sa_session, account_id=self.account_id)
        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, tmp_file.file_hash)

        if importer.get_row_count(file_path) > importer.MAX_RECORDS_FOR_INSTANT_IMPORT:
            importer.import_file(tmp_file, importer, tbs_id=tbs_id, regulation_period_id=regulation_period_id)
            self.wpc.start_task('we.product.schedule.CN8ToTbsProductLevelTask', {})
            return False

        return self.wpc.import_cn8_to_tbs_product_level(tmp_file, importer, tbs_id, regulation_period_id)

    def mark_as_deleted(self, obj, categories):
        return self.wpc.mark_as_deleted(obj, categories)

    def mark_as_archived(self, obj, categories):
        return self.wpc.mark_tbs_product_levels_as_archived(obj, categories)

    def archive_regulation_period(self, rp_id):
        return self.wpc.archive_regulation_period(rp_id)

    def validate_cn8_code_form(self, args):
        form_handler = FormValidator({
            'cn8_code': [(plugins.Required, {}), (plugins.Number, {})],
        })

        return form_handler.validate(args)

    def validate_local_category_form(self, args):
        form_handler = FormValidator({
            'tbs_id': [(plugins.Required, {})],
            'category': [(plugins.Required, {})],
            'subcategory': [(plugins.Required, {})],
            'category_label': [(plugins.Required, {})],
            'subcategory_label': [(plugins.Required, {})],
        })

        return form_handler.validate(args)

    def validate_cn8_to_subcategory_form(self, args):
        form_handler = FormValidator({
            'cn8_codes': [(plugins.Required, {})],
            'unu_code': [(plugins.Required, {})],
            'weee_product_level_id': [(plugins.Required, {})],
        })

        return form_handler.validate(args)

    def validate_hqpp_product_form(self, args):
        form_handler = FormValidator({
            'cn8_codes': [(plugins.Required, {})],
            'sku': [(plugins.Required, {})],
            'product_name': [(plugins.Required, {})],
            'weight_per_unit': [(plugins.Required, {})],
            'hqpp_id': [(plugins.Required, {})]
        })

        return form_handler.validate(args)

    def validate_tbs_product_level_details_form(self, args, is_main=False):

        if is_main:
            validate = {
                'description': [(plugins.Required, {})],
                'label': [(plugins.Required, {})],
                'invoice_basis': [(plugins.Required, {})],
                'tbs_id': [(plugins.Required, {})],
                'tbs_regulation_period_id': [(plugins.Required, {})]
            }
        else:
            validate = {
                'description': [(plugins.Required, {})],
                'label': [(plugins.Required, {})],
                'invoice_basis': [(plugins.Required, {})],
                'parent_level_id': [(plugins.Required, {})],
                'tbs_id': [(plugins.Required, {})],
                'tbs_regulation_period_id': [(plugins.Required, {})]
            }

        form_handler = FormValidator(validate)

        return form_handler.validate(args)

    def validate_tbs_regulation_period(self, args):

        validate = {
            'tbs_id': [(plugins.Required, {})],
            'label': [(plugins.Required, {})],
            'start_year': [(plugins.Required, {})],
            'start_month': [(plugins.Required, {})],
            'tbs_product_levels': [(plugins.Required, {})],
            'weight_settings': [(plugins.Required, {})],
        }

        form_handler = FormValidator(validate)

        return form_handler.validate(args)

    def validate_new_tbs_price_sheet(self, args):
        validate = {
            'tbs_id': [(plugins.Required, {})],
            'regulation_label': [(plugins.Required, {})],
            'regulation_year': [(plugins.Required, {})],
            'regulation_month': [(plugins.Required, {})],
            'upload_price_sheet_new': [(plugins.Required, {})],
            'weight_settings': [(plugins.Required, {})],
        }

        form_handler = FormValidator(validate)

        return form_handler.validate(args)

    def validate_hqpp_product_wizard_page(self, args):
        form_handler = FormValidator({
            'tbs_product_level_id': [(plugins.Required, {})]
            })

        return form_handler.validate(args)

    def validate_hqpp_product_sheet_form(self, args):
        form_handler = FormValidator({
            'hqpp_id': [(plugins.Required, {})],
            'upload_products': [(plugins.Required, {})],
        })

        return form_handler.validate(args)

def export_hqpp_product_sheet(request, hqpp_id):
        """
        Writes data to excel file. User for Customdatatable data export.
        """
        sa_session = request.Session()

        wpc = WeProductController(sa_session, account_id=request.session['user_id'], request=request)
        f_name = wpc.export_hqpp_product_sheet(int(hqpp_id))

        response = HttpResponse(open(f_name, 'r'), mimetype='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
        response['Content-Disposition'] = 'attachment; filename=%s' % f_name

        return response


handlers = (WeProductRpcHandler, )
