# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iqube.core.db_sync import AbstractConfigurableWidget
from django.utils.translation import ugettext_lazy as _

class WeDeclarationResultWidget(AbstractConfigurableWidget):
    category = 'Report'
    cube = 'WeDeclarationDataCube'
    datasource = 'WeDeclarationData'
    label = 'Declaration result'

    def config(self):
        return {
            "reporter": "we.report.WeDeclarationReporter",
            'show_last_aggregation_update': True,
            "category": "Declarations",
            "has_quick_filter": True,
            "show_hide_empty": True,
            "transposable": True,
            "label": "Declaration report",
            "has_calculated_member": True,
            "show_excel_export": True,
            "cube": "WeDeclarationDataCube",
            "permissions": ['we_view_report'],
            "has_graph": True,
            "table_object_name": "datatablecube",
            "table_module": "iqreport_graph_datatablecube",
            'filter_panel_module': 'we_report_declaration_item',
            "has_table": True,
            "has_hierarchy_names": True,
            "graph_config": {},

            "has_drill_position": False,
            "has_drill_replace": False,

            "has_drill_through": False,

            "dimensions": [{
                    "caption": "TBS",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[TBS].[TBS]",
                            "has_children_available": True,
                            "caption": "TBS",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[TBS].[TBS]",
                            "method": "get_level_members"
                        }]
                },  {
                    "caption": "Producer",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "fancy_select_optionsRpcUrl": 'we.report.ReportDimensionHandler',
                    "fancy_select_RpcMethod": 'get_producers',
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Producer].[Producer]",
                            "has_children_available": True,
                            "caption": "Member",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Producer].[Producer]",
                            "method": "get_level_members"
                    }]
                } ,{
                    "caption": "LPP",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "fancy_select_optionsRpcUrl": 'we.report.ReportDimensionHandler',
                    "fancy_select_RpcMethod": 'get_lpps',
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[LPP].[LPP]",
                            "has_children_available": True,
                            "caption": "LPP",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[LPP].[LPP]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Country",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Country].[Country]",
                            "has_children_available": True,
                            "caption": "Country",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Country].[Country]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Declaration year",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Declaration Year].[Year]",
                            "has_children_available": True,
                            "caption": "Year",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Declaration Year].[Year]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Declaration month",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Declaration Month].[Month]",
                            "has_children_available": True,
                            "caption": "Month",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Declaration Month].[Month]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Declaration status",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Declaration Status].[Declaration Status]",
                            "has_children_available": True,
                            "caption": "Declaration Status",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Declaration Status].[Declaration Status]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Product",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Product].[Product]",
                            "has_children_available": True,
                            "caption": "Product",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Product].[Product]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Local category",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Local Category].[Local Category]",
                            "has_children_available": True,
                            "caption": "Local Category",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Local Category].[Local Category]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "CN8",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[CN8].[CN8]",
                            "has_children_available": True,
                            "caption": "CN8",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[CN8].[CN8]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "WEEE category",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[WEEE Category].[WEEE Category]",
                            "has_children_available": True,
                            "caption": "WEEE Category",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[WEEE Category].[WEEE Category]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Measures",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": True,
                    "children": [{
                            "unique_name": "[Measures].[MeasuresLevel]",
                            "has_children_available": True,
                            "caption": "Measures",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Measures].[MeasuresLevel]",
                            "method": "get_level_members"
                    }]
                }
            ],
        }

    def applied_filters(self):
        return {
            "dimensions": {
                "TBS": "0",
                "Producer": "1",
                "LPP": "2",
                "Country":"3",
                "Declaration year": "4",
                "Declaration month": "5",
                "Declaration status": "6",
                "Product": "7",
                "Local category": "8",
                "CN8": "9",
                "WEEE Category": "10",
                "Measures": "11"
            },

            "axes_0": "",
            "axes_1": "",
            "axes_2": "",
            "axes_3": "",
            "axes_4": "",
            "axes_5": "",
            "axes_6": "",
            "axes_7": "",
            "axes_8": "",
            "axes_9": "",
            "axes_10":"",
            "axes_11": "columns",


            "dimension_0": [],
            "dimension_1": [],
            "dimension_2": [],
            "dimension_3": [],
            "dimension_4": [],
            "dimension_5": [],
            "dimension_6": [],
            "dimension_7": [],
            "dimension_8": [],
            "dimension_9": [],
            "dimension_10":[],
            "dimension_11": ["[Measures].[Weight]"],

            "column_order": [
                "Measures",
            ],
            "row_order": [
            ],
            "show_table": True,
            "show_slicers": True,
            "hide_empty_fields": True,

            "initialRefresh": False,

        }

configurable_widgets = (WeDeclarationResultWidget, )
