# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.
from iqportal.iqube.core.db_sync import AbstractConfigurableWidget
from we.report.iqube_sync import WeOrderCube

from django.utils.translation import ugettext as _

class WeOrderResultWidget(AbstractConfigurableWidget):
    cube_object = WeOrderCube
    cube = 'WeOrderCube'
    category = 'Report'
    datasource = 'WeDeclarationData'
    label = _('Order result')

    def config(self):
        return {
            "reporter": "we.report.WeDeclarationReporter",
            'show_last_aggregation_update': True,
            "category": "Orders",
            "has_quick_filter": True,
            "show_hide_empty": True,
            "transposable": True,
            "label": "Order report",
            "has_calculated_member": True,
            "show_excel_export": True,
            "cube": "WeOrderCube",
            "permissions": ['we_view_report'],
            "has_graph": True,
            "table_object_name": "datatablecube",
            "table_module": "iqreport_graph_datatablecube",
            'filter_panel_module': 'we_report_declaration_item',
            "has_table": True,
            "has_hierarchy_names": True,
            "graph_config": {},

            "has_drill_position": False,
            "has_drill_replace": False,

            "has_drill_through": False,

            "dimensions": [{
                    "caption": "TBS",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "fancy_select_optionsRpcUrl": 'we.report.ReportDimensionHandler',
                    "fancy_select_RpcMethod": 'get_tbses',
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[TBS].[TBS]",
                            "has_children_available": True,
                            "caption": "TBS",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[TBS].[TBS]",
                            "method": "get_level_members"
                        }]
                },{
                    "caption": "Producer",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "fancy_select_optionsRpcUrl": 'we.report.ReportDimensionHandler',
                    "fancy_select_RpcMethod": 'get_producers',
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Producer].[Producer]",
                            "has_children_available": True,
                            "caption": "Member",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Producer].[Producer]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "LPP",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "fancy_select_optionsRpcUrl": 'we.report.ReportDimensionHandler',
                    "fancy_select_RpcMethod": 'get_lpps',
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[LPP].[LPP]",
                            "has_children_available": True,
                            "caption": "LPP",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[LPP].[LPP]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Country",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Country].[Country]",
                            "has_children_available": True,
                            "caption": "Country",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Country].[Country]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Order Year",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Order Year].[Year]",
                            "has_children_available": True,
                            "caption": "Year",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Order Year].[Year]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Order Bi-annual",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Order Bi-annual].[Bi-annual]",
                            "has_children_available": True,
                            "caption": "Bi-annual",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Order Bi-annual].[Bi-annual]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Order Quarter",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Order Quarter].[Quarter]",
                            "has_children_available": True,
                            "caption": "Quarter",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Order Quarter].[Quarter]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Order Month",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Order Month].[Month]",
                            "has_children_available": True,
                            "caption": "Month",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Order Month].[Month]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Local category",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "fancy_select_optionsRpcUrl": 'we.report.ReportDimensionHandler',
                    "fancy_select_RpcMethod": 'get_local_categories',
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Local Category].[Local Category]",
                            "has_children_available": True,
                            "caption": "Local Category",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Local Category].[Local Category]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "B2B/B2C",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Is b2b].[Is b2b]",
                            "has_children_available": True,
                            "caption": "Is b2b",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Is b2b].[Is b2b]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Measures",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": True,
                    "children": [{
                            "unique_name": "[Measures].[MeasuresLevel]",
                            "has_children_available": True,
                            "caption": "Measures",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Measures].[MeasuresLevel]",
                            "method": "get_level_members"
                    }]
                }
            ],
        }

    def applied_filters(self):
        return {
            "dimensions": {
                "TBS": "0",
                "Producer": "1",
                "LPP": "2",
                "Country": "3",
                "Order Year": "4",
                "Order Bi-annual": "5",
                "Order Quarter": "6",
                "Order Month": "7",
                "Local Category": "8",
                "Is b2b": "9",
                "Measures": "10"
            },

            "axes_0": "",
            "axes_1": "",
            "axes_2": "",
            "axes_3": "",
            "axes_4": "",
            "axes_5": "",
            "axes_6": "",
            "axes_7": "",
            "axes_8": "",
            "axes_9": "",
            "axes_10": "columns",

            "dimension_0": [],
            "dimension_1": [],
            "dimension_2": [],
            "dimension_3": [],
            "dimension_4": [],
            "dimension_5": [],
            "dimension_6": [],
            "dimension_7": [],
            "dimension_8": [],
            "dimension_9": [],
            "dimension_10": ["[Measures].[Weight]"],

            "column_order": [
                "Measures",
            ],
            "row_order": [
            ],
            "show_table": True,
            "show_slicers": True,
            "hide_empty_fields": True,

            "initialRefresh": False,

        }


configurable_widgets = (WeOrderResultWidget,)
