# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

import datetime

from sqlalchemy import Integer, Column, DateTime, SmallInteger, DECIMAL, String

from basesoft.web.contrib.license import models as license_models
from basesoft.sqlalchemy.table import IQTable

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    aggWeDeclaration = IQTable('aggWeDeclaration', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('declaration_id', Integer, index=True, nullable=False),
        Column('declaration_line_id', Integer, index=True, nullable=False),
        Column('declaration_status', Integer, index=True, nullable=False),
        Column('lpp_id', Integer, index=True, nullable=False),
        Column('tbs_id', Integer, index=True, nullable=False),
        Column('hqpp_id', Integer, index=True, nullable=False),
        Column('country_id', Integer, index=True, nullable=False),
        Column('year', SmallInteger, index=True, nullable=False),
        Column('month', SmallInteger, index=True, nullable=False),
        Column('hqpp_period_product_id', Integer, index=True, nullable=False),
        Column('tbs_product_level_id', Integer, index=True, nullable=False),
        Column('weee_product_level_id', Integer, index=True, nullable=False),
        Column('cn8', Integer, index=True, nullable=False),
        Column('unu_code_id', Integer, index=True, nullable=True),
        Column('quantity', Integer, index=True, nullable=False),
        Column('weight', DECIMAL(20,5), index=True, nullable=False),
        Column('amount_excluding_vat', DECIMAL(20,5), index=True, nullable=False),
        Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    aggWeOrder = IQTable('aggWeOrder', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('order_id', Integer, index=True, nullable=False),
        Column('order_line_id', Integer, index=True, nullable=False),
        Column('tbs_id', Integer, index=True, nullable=False),
        Column('hqpp_id', Integer, index=True, nullable=False),
        Column('lpp_id', Integer, index=True, nullable=False),
        Column('country_id', Integer, index=True, nullable=False),
        Column('year', SmallInteger, index=True, nullable=False),
        Column('biannual', SmallInteger, index=True, nullable=True),
        Column('quarter', SmallInteger, index=True, nullable=True),
        Column('month', SmallInteger, index=True, nullable=True),
        Column('tbs_product_level_id', Integer, index=True, nullable=False),
        Column('is_b2b', Integer, index=True, nullable=True),
        Column('quantity', Integer, index=True, nullable=False),
        Column('weight', DECIMAL(20,3), index=True, nullable=False),
        Column('line_amount_excluding_vat', DECIMAL(20,2), index=True, nullable=False),
        Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    tables['aggWeDeclaration'] = aggWeDeclaration
    tables['aggWeOrder'] = aggWeOrder

    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'WEEE Europe report')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [
        license_models.Permission(u'we_view_report', u"""
            This gives the right to view reports
            """, _("View reports")),
    ]
