'''
Created on Jan 29, 2015
Copyright (c) 2016 BaseSoft B.V.
@author: marten
'''


from django.utils.translation import ugettext, ugettext_lazy as _
from basesoft.web.core.config import DefaultConfig
from basesoft.web.core.input.validation import plugins
from we.tbs.models import WeTbsLicense


class WeTbsLicenseConfigs(DefaultConfig):
    permissions = ('we_tbs_license', )
    label = _('TBS settings')
    is_expanded = True

    def configs(self):
        return [{
            'name': 'we.tbs.declaration.period',
            'label': ugettext('Maximum period in which a declaration must be performed'),
            'options': {
                'options': {
                    WeTbsLicense.DECLARATION_PERIOD_MONTHLY: ugettext('Monthly'),
                    WeTbsLicense.DECLARATION_PERIOD_QUARTERLY: ugettext('Quarterly'),
                    WeTbsLicense.DECLARATION_PERIOD_BIANNUALLY: ugettext('Bi-annually'),
                    WeTbsLicense.DECLARATION_PERIOD_YEARLY: ugettext('Yearly'),
                }
            }
        }, {
            'name': 'we.tbs.use.sales.price',
            'label': ugettext('Swiss TBSes use sales price to allocate SKU into a local category'),
        }, {
            'name': 'we.tbs.require.end.user.distinction',
            'label': ugettext('Member needs to provide a TBS category distinction between b2b and b2c'),
        }, {
            'name': 'we.tbs.use.negative.fee',
            'label': ugettext('Enable negative fee'),
        }, {
            'name': 'we.tbs.declaration.closing.day',
            'label': ugettext('Declaration closing day'),
            'help': ugettext('Maximum usable day depends on the periodicity; Monthly: %(m)s, Quarterly: %(q)s, Bi-annually: %(b)s, Yearly: %(y)s' % {
                'm': 27,
                'q': 89,
                'b': 180,
                'y': 364,
            })
        }, {
            'name': 'we.tbs.declaration.reminder.day',
            'label': ugettext('Declaration reminder day'),
            'help': ugettext('Maximum usable day depends on the periodicity; Monthly: %(m)s, Quarterly: %(q)s, Bi-annually: %(b)s, Yearly: %(y)s' % {
                'm': 26,
                'q': 88,
                'b': 179,
                'y': 363,
            })
        }, {
            'name': 'we.tbs.is.partner',
            'label': ugettext('TBS is partner'),
        }, {
            'name': 'basesoft.web.contrib.order.export_zero_amount_orders',
            'label': ugettext('Do zero amount orders generate an invoice?'),
            'help': ugettext('If the total amount of an order is 0, an invoice will still be generated when enabled.')
        }]

    def get_form_behaviour(self, validators, values):

        closing_day_integer_params = {'unsigned': True, 'allow_zero': False}
        reminder_day_integer_params = {'unsigned': True, 'allow_zero': False}
        if not values.has_key('we.tbs.declaration.period') or values['we.tbs.declaration.period'] == None:
            validators['we.tbs.declaration.closing.day'] = [(plugins.Integer, closing_day_integer_params, )]
            return validators

        p = int(values['we.tbs.declaration.period'])
        if p == WeTbsLicense.DECLARATION_PERIOD_MONTHLY:
            closing_day_integer_params['max_value'] = 28 - 1  # february

        if p == WeTbsLicense.DECLARATION_PERIOD_QUARTERLY:
            closing_day_integer_params['max_value'] = 31 + 28 + 31 - 1  # 89 -> quarter 1 is the shortest

        if p == WeTbsLicense.DECLARATION_PERIOD_BIANNUALLY:
            closing_day_integer_params['max_value'] = 31 + 28 + 31 + 30 + 31 + 30 - 1  #  180 -> first half year is the shortest

        if p == WeTbsLicense.DECLARATION_PERIOD_YEARLY:
            closing_day_integer_params['max_value'] = 365 - 1

        reminder_day_integer_params['max_value'] = closing_day_integer_params['max_value'] - 1
        validators['we.tbs.declaration.closing.day'] = [(plugins.Integer, closing_day_integer_params, )]
        validators['we.tbs.declaration.reminder.day'] = [(plugins.Integer, reminder_day_integer_params, )]

        return validators


license_configs = {
    'WeTbsLicense': (WeTbsLicenseConfigs(), ),
}
