'''
Created on Dec 17, 2014

@author: marten
'''
from sqlalchemy import func
from django.utils.translation import ugettext_lazy as _

from basesoft.web.core.util.custom_query import CustomQueryModel, CustomQueryField, DATA_TYPE_BOOLEAN
from basesoft.web.contrib.license.custom_query import LicenseCustomQuery, TempCompany
from basesoft.web.contrib.license.models import Domain
from basesoft.web.contrib.i18n.models import Country
from basesoft.web.contrib.contact.models import Company, Person, CompanyAddress, Address

from we.core.custom_query import AbstractWeCustomQuery
from we.tbs.models import WeTbsLicense

class TbsCustomQuery(AbstractWeCustomQuery, LicenseCustomQuery):
    """
    We extend the License Custom Query for additional permission checks.
    """

    def validate_access(self):
        self.assert_any_permissions('we_tbs_license')
        LicenseCustomQuery.validate_access(self)

    def tables(self):
        return (
            CustomQueryModel(WeTbsLicense),
            CustomQueryModel(Domain, onclause=WeTbsLicense.domain_id==Domain.id),
            CustomQueryModel(Company, onclause=WeTbsLicense.company_id==Company.id, use_outerjoin=True),
            CustomQueryModel(CompanyAddress, use_outerjoin=True),
            CustomQueryModel(Address, use_outerjoin=True),
            CustomQueryModel(Country, onclause=Address.country_id==Country.id, use_outerjoin=True),
            CustomQueryModel(Person, onclause=Company.contact_person_id==Person.id),
            CustomQueryModel(TempCompany, onclause=WeTbsLicense.temp_company_id==TempCompany.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(WeTbsLicense.id, _('ID'), key='license_id', is_filter=True),
            CustomQueryField(WeTbsLicense.name, _('Name'), key='license_name', is_filter=True),
            CustomQueryField(WeTbsLicense.id, _('Settings'), key='settings'),
            CustomQueryField(WeTbsLicense.is_active, _('Active'), is_filter=True, key='is_active', type=DATA_TYPE_BOOLEAN),
            CustomQueryField(Country.name, _('Country'), is_filter=True, key='country'),
            CustomQueryField(func.concat_ws(' ', Person.first_name, Person.middle_name, Person.last_name), _('Contact'), key='contact'),
            CustomQueryField(Company.name, _('Company'), 'company_name'),
            CustomQueryField(Company.notes, _('Notes'), key='notes'),
            CustomQueryField(TempCompany.is_temp, _('Company details change'), key='master_data', is_filter=True),
        )

    group_by = (WeTbsLicense.license_id, )

custom_queries = (
    TbsCustomQuery,
)
